@@ -1,6 +1,7 @@
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::node::{CascadedValues, NodeResult, NodeTrait, RsvgNode};
+use crate::element::{ElementResult, SetAttributes};
+use crate::node::{CascadedValues, Node};
 use crate::property_bag::PropertyBag;
 
 use super::context::{FilterContext, FilterOutput, FilterResult};
@@ -21,33 +22,33 @@ impl Default for FeFlood {
     }
 }
 
-impl NodeTrait for FeFlood {
-    impl_node_as_filter_effect!();
-
-    #[inline]
-    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
-        self.base.set_atts(parent, pbag)
+impl SetAttributes for FeFlood {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
+        self.base.set_attributes(pbag)
     }
 }
 
 impl FilterEffect for FeFlood {
     fn render(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         ctx: &FilterContext,
         _acquired_nodes: &mut AcquiredNodes,
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
-        let bounds = self.base.get_bounds(ctx).into_irect(draw_ctx);
+        let bounds = self
+            .base
+            .get_bounds(ctx, node.parent().as_ref())?
+            .into_irect(draw_ctx);
 
         let cascaded = CascadedValues::new_from_node(node);
         let values = cascaded.get();
 
-        let color = match values.flood_color.0 {
-            cssparser::Color::CurrentColor => values.color.0,
+        let color = match values.flood_color().0 {
+            cssparser::Color::CurrentColor => values.color().0,
             cssparser::Color::RGBA(rgba) => rgba,
         };
-        let opacity = values.flood_opacity.0;
+        let opacity = values.flood_opacity().0;
 
         let surface = ctx.source_graphic().flood(bounds, color, opacity)?;
 
