@@ -1,5 +1,6 @@
 //! Lighting filters and light nodes.
 
+use float_cmp::approx_eq;
 use markup5ever::{expanded_name, local_name, namespace_url, ns};
 use matches::matches;
 use nalgebra::{Vector2, Vector3};
@@ -422,7 +423,7 @@ impl Lighting for FeSpecularLighting {
         let k = if normal.normal.is_zero() {
             // Common case of (0, 0, 1) normal.
             let n_dot_h = h.z / h_norm;
-            if (self.specular_exponent - 1.0).abs() < f64::EPSILON {
+            if approx_eq!(f64, self.specular_exponent, 1.0) {
                 n_dot_h
             } else {
                 n_dot_h.powf(self.specular_exponent)
@@ -435,7 +436,7 @@ impl Lighting for FeSpecularLighting {
             let normal = Vector3::new(n.x, n.y, 1.0);
 
             let n_dot_h = normal.dot(&h) / normal.norm() / h_norm;
-            if (self.specular_exponent - 1.0).abs() < f64::EPSILON {
+            if approx_eq!(f64, self.specular_exponent, 1.0) {
                 n_dot_h
             } else {
                 n_dot_h.powf(self.specular_exponent)
