@@ -1,4 +1,47 @@
 //! Definitions for CSS property types.
+//!
+//! This module defines most of the CSS property types that librsvg supports.  Each
+//! property requires a Rust type that will hold its values, and that type should
+//! implement a few traits, as follows.
+//!
+//! # Requirements for a property type
+//!
+//! You should call the [`make_property`] macro to take care of most of these requirements
+//! automatically:
+//!
+//! * A name for the type.  For example, the `fill` property has a [`Fill`] type defined
+//! in this module.
+//!
+//! * An initial value per the CSS or SVG specs, given through an implementation of the
+//! `Default` trait.
+//!
+//! * Whether the property's computed value inherits to child elements, given
+//! through an implementation of the [`inherits_automatically`] method of the [`Property`]
+//! trait.
+//!
+//! * A way to derive the CSS *computed value* for the property, given through an
+//! implementation of the [`compute`] method of the [`Property`] trait.
+//!
+//! * The actual underlying type.  For example, the [`make_property`] macro can generate a
+//! field-less enum for properties like the `clip-rule` property, which just has
+//! identifier-based values like `nonzero` and `evenodd`.  For general-purpose types like
+//! [`Length`], the macro can wrap them in a newtype like `struct`
+//! [`StrokeWidth`]`(Length)`.  For custom types, the macro call can be used just to
+//! define the initial/default value and whether the property inherits automatically; you
+//! should provide the other required trait implementations separately.
+//!
+//! * An implementation of the [`Parse`] trait for the underlying type.
+//!
+//! [`compute`]: ../property_macros/trait.Property.html#tymethod.compute
+//! [`inherits_automatically`]: ../property_macros/trait.Property.html#tymethod.inherits_automatically
+//! [`Fill`]: struct.Fill.html
+//! [`Length`]: ../length/struct.Length.html
+//! [`make_property`]: ../macro.make_property.html
+//! [`Opacity`]: struct.Opacity.html
+//! [`Parse`]: ../trait.Parse.html
+//! [`Property`]: ../property_macros/trait.Property.html
+//! [`UnitInterval`]: ../unit_interval/struct.UnitInterval.html
+use std::convert::TryInto;
 
 use cssparser::{Parser, Token};
 
@@ -407,6 +450,32 @@ make_property!(
     newtype_parse: IRI,
 );
 
+// https://www.w3.org/TR/compositing/#mix-blend-mode
+make_property!(
+    ComputedValues,
+    MixBlendMode,
+    default: Normal,
+    inherits_automatically: false,
+
+    identifiers:
+    "normal" => Normal,
+    "multiply" => Multiply,
+    "screen" => Screen,
+    "overlay" => Overlay,
+    "darken" => Darken,
+    "lighten" => Lighten,
+    "color-dodge" => ColorDodge,
+    "color-burn" => ColorBurn,
+    "hard-light" => HardLight,
+    "soft-light" => SoftLight,
+    "difference" => Difference,
+    "exclusion" => Exclusion,
+    "hue" => Hue,
+    "saturation" => Saturation,
+    "color" => Color,
+    "luminosity" => Luminosity,
+);
+
 // https://www.w3.org/TR/SVG/masking.html#OpacityProperty
 make_property!(
     ComputedValues,
@@ -430,6 +499,90 @@ make_property!(
     "auto" => Auto,
 );
 
+#[repr(u8)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq)]
+pub enum PaintTarget {
+    Fill,
+    Stroke,
+    Markers,
+}
+
+// https://www.w3.org/TR/SVG2/painting.html#PaintOrder
+make_property!(
+    ComputedValues,
+    PaintOrder,
+    inherits_automatically: true,
+    fields: {
+        targets: [PaintTarget; 3], default: [PaintTarget::Fill, PaintTarget::Stroke, PaintTarget::Markers],
+    }
+
+    parse_impl: {
+        impl Parse for PaintOrder {
+            fn parse<'i>(parser: &mut Parser<'i, '_>) -> Result<PaintOrder, ParseError<'i>> {
+                let allowed_targets = 3;
+                let mut targets = Vec::with_capacity(allowed_targets);
+
+                if parser.try_parse(|p| p.expect_ident_matching("normal")).is_ok() {
+                    return Ok(PaintOrder::default());
+                }
+
+                while !parser.is_exhausted() {
+                    let loc = parser.current_source_location();
+                    let token = parser.next()?;
+
+                    let value = match token {
+                        Token::Ident(ref cow) if cow.eq_ignore_ascii_case("fill") && !targets.contains(&PaintTarget::Fill) => PaintTarget::Fill,
+                        Token::Ident(ref cow) if cow.eq_ignore_ascii_case("stroke") && !targets.contains(&PaintTarget::Stroke) => PaintTarget::Stroke,
+                        Token::Ident(ref cow) if cow.eq_ignore_ascii_case("markers") && !targets.contains(&PaintTarget::Markers) => PaintTarget::Markers,
+                        _ => return Err(loc.new_basic_unexpected_token_error(token.clone()).into()),
+                    };
+
+                    targets.push(value);
+                };
+
+                // any values which were not specfied should be painted in default order
+                // (fill, stroke, markers) following the values which were explicitly specified.
+                for &target in &[PaintTarget::Fill, PaintTarget::Stroke, PaintTarget::Markers] {
+                    if !targets.contains(&target) {
+                        targets.push(target);
+                    }
+                }
+                Ok(PaintOrder {
+                    targets: targets[..].try_into().expect("Incorrect number of targets in paint-order")
+                })
+            }
+        }
+    }
+);
+
+#[cfg(test)]
+#[test]
+fn parses_paint_order() {
+    assert_eq!(
+        PaintOrder::parse_str("normal").unwrap(),
+        PaintOrder {
+            targets: [PaintTarget::Fill, PaintTarget::Stroke, PaintTarget::Markers]
+        }
+    );
+
+    assert_eq!(
+        PaintOrder::parse_str("markers fill").unwrap(),
+        PaintOrder {
+            targets: [PaintTarget::Markers, PaintTarget::Fill, PaintTarget::Stroke]
+        }
+    );
+
+    assert_eq!(
+        PaintOrder::parse_str("stroke").unwrap(),
+        PaintOrder {
+            targets: [PaintTarget::Stroke, PaintTarget::Fill, PaintTarget::Markers]
+        }
+    );
+
+    assert!(PaintOrder::parse_str("stroke stroke").is_err());
+    assert!(PaintOrder::parse_str("markers stroke fill hello").is_err());
+}
+
 // https://www.w3.org/TR/SVG/painting.html#ShapeRenderingProperty
 make_property!(
     ComputedValues,
