@@ -2,8 +2,8 @@
 
 use markup5ever::{expanded_name, local_name, namespace_url, ns};
 
+use crate::element::{Draw, ElementResult, SetAttributes};
 use crate::error::*;
-use crate::node::{NodeResult, NodeTrait, RsvgNode};
 use crate::property_bag::PropertyBag;
 
 /// Represents the syntax used in the <style> node.
@@ -50,8 +50,8 @@ impl Style {
     }
 }
 
-impl NodeTrait for Style {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Style {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             if attr.expanded() == expanded_name!("", "type") {
                 self.type_ = Some(StyleType::parse(value).attribute(attr)?);
@@ -62,6 +62,8 @@ impl NodeTrait for Style {
     }
 }
 
+impl Draw for Style {}
+
 #[cfg(test)]
 mod tests {
     use super::*;
